package com.ruoyi.web;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.domain.ReviewSceneChangeTask;
import com.ruoyi.service.ReviewSceneChangeTaskService;
import com.ruoyi.web.request.ReviewSceneChangeTaskCreateRequest;
import com.ruoyi.web.request.ReviewSceneChangeTaskPassRequest;
import org.apache.skywalking.apm.toolkit.trace.Tag;
import org.apache.skywalking.apm.toolkit.trace.Tags;
import org.apache.skywalking.apm.toolkit.trace.Trace;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/review/scene/change/task")
public class ReviewSceneChangeTaskController extends BaseController {

    @Autowired
    private ReviewSceneChangeTaskService reviewSceneChangeTaskService;

    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "场景变更任务", businessType = BusinessType.INSERT)
    @RequestMapping(method = RequestMethod.POST, value = "/create")
    public AjaxResult create(@Validated @RequestBody ReviewSceneChangeTaskCreateRequest request) {

        ReviewSceneChangeTask reviewSceneChangeTask = new ReviewSceneChangeTask();
        BeanUtils.copyBeanProp(reviewSceneChangeTask, request);
        reviewSceneChangeTaskService.create(reviewSceneChangeTask);

        return success();
    }

    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "审核通过任务", businessType = BusinessType.UPDATE)
    @RequestMapping(method = RequestMethod.POST, value = "/pass")
    public AjaxResult pass(@Validated @RequestBody ReviewSceneChangeTaskPassRequest request) {

        reviewSceneChangeTaskService.pass(request.getTaskId(),request.getComment());

        return success();
    }

    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "审核驳回任务", businessType = BusinessType.UPDATE)
    @RequestMapping(method = RequestMethod.POST, value = "/reject")
    public AjaxResult reject(@Validated @RequestBody ReviewSceneChangeTaskPassRequest request) {

        reviewSceneChangeTaskService.reject(request.getTaskId(),request.getComment());

        return success();
    }

    /**
     * 根据用户ID查询待办列表
     * @return
     */
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/findPending")
    public TableDataInfo findPending() {
        startPage();
        return getDataTable(reviewSceneChangeTaskService.findPendingListByUserId(getUserId()));
    }

    /**
     * 根据用户ID查询已办列表
     * @return
     */
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/findFinish")
    public TableDataInfo findFinish() {
        startPage();
        return getDataTable(reviewSceneChangeTaskService.findFinishListByUserId(getUserId()));
    }

    /**
     * 查询当前用户发布的任务
     * @return
     */
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/findCreate")
    public TableDataInfo findCreate() {
        startPage();
        return getDataTable(reviewSceneChangeTaskService.findCreateListByUserId(getUserId()));
    }

}
